﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;
using static System.Windows.Forms.VisualStyles.VisualStyleElement.TaskbarClock;
using System.Xml.Linq;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json;
using System.Net.Http;
using System.Globalization;
using System.Drawing;
using Aspose.Words;
using System.IO;
using System.Threading;
using static System.Net.Mime.MediaTypeNames;
using System.Drawing.Drawing2D;

namespace travelExport
{
    public partial class Home : Form
    {
        readonly static oa2023DBEntities db = new oa2023DBEntities();
        static List<Grp_DelegationInfo> soure = new List<Grp_DelegationInfo>();
        readonly static string[] weekdays = new string[] { "星期日", "星期一", "星期二", "星期三", "星期四", "星期五", "星期六" };
        const decimal conversion = 1.61M;
        readonly static string[] excludeArr = new string[] { "[中转]", "[转机]" };

        public Home()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            soure =  db.Grp_DelegationInfo.Where(x => x.IsDel == 0).ToList();
            comb_Delegation.ValueMember = "Id";
            comb_Delegation.DisplayMember = "TeamName";
            comb_Delegation.Items.AddRange(soure.ToArray());
            comb_Delegation.DropDownHeight = 200;
            comb_Delegation.MouseClick += comb_Delegation_MouseClick;
            comb_Delegation.TextUpdate += comb_Delegation_TextUpdate;
            FormBorderStyle = FormBorderStyle.FixedSingle;
            MinimizeBox = false;
            MaximizeBox = false;
        }

        private void comb_Delegation_MouseClick(object sender, MouseEventArgs e)
        {
            comb_Delegation.Text = "";
            comb_Delegation_TextUpdate(sender, e);
            comb_Delegation.DropDownHeight = 200;
            comb_Delegation.DroppedDown = true;
        }

        private void comb_Delegation_TextUpdate(object sender, EventArgs e)
        {
            ComboBox cb = (ComboBox)sender;
            cb.DataSource = null;
            cb.Items.Clear();
            string s = cb.Text;
            var bindList = soure.Where(item => item.TeamName != null && item.TeamName.Contains(s)).ToList();
            if (bindList.Count >= 1) // 存在符合条件的内容
            {
                this.comb_Delegation.Items.AddRange(bindList.ToArray());
            }
            else
            {
                this.comb_Delegation.Items.Add(new Grp_DelegationInfo { TeamName = "", Id = -1 });
            }

            cb.SelectionStart = cb.Text.Length;  // 设置光标位置，若不设置：光标位置始终保持在第一列，造成输入关键词的倒序排列
            cb.Cursor = Cursors.Default; //保持鼠标指针原来状态，有时候鼠标指针会被下拉框覆盖，所以要进行一次设置
            cb.MaxDropDownItems = 8; // 自动弹出下拉框            
        }

        private async void btnImport_Click(object sender, EventArgs e)
        {
            var diid = -1;
            try
            {
                var dele = (comb_Delegation.SelectedItem as Grp_DelegationInfo);
                diid = dele.Id;
            }
            catch (Exception)
            {
                //lblImport.Text = "请正确的选择团组!";
                //lblImport.ForeColor = Color.Black;
                MessageBoxEx.Show(this,"请正确的选择团组!","提示");
                return;
            }

            //GetAllCity(diid);

            DataTable resultTable = null;
            try
            {
                resultTable = GetTableByBlackCode(diid);
            }
            catch (Exception ex)
            {
                MessageBoxEx.Show(this, "黑屏代码有误!", "提示");
                return;
            }

            if (resultTable.Rows.Count == 0 || string.IsNullOrWhiteSpace(resultTable.Rows[0][1].ToString()))
            {
                string msg = string.Empty;
                if (resultTable.Rows.Count > 0)
                {
                    msg = resultTable.Rows[0]["Error"].ToString();
                }
                MessageBoxEx.Show(this, "黑屏代码有误! " + msg , "提示");
                return;
            }

            var isVPN =  await Task.Run(() =>
            {
                var isVpn = GetDirectionByGoogleApi("北京", "上海");
                if (Convert.ToInt32(isVpn["code"]) == -1)
                {
                    return false;
                }
                return true;
            });

            if (!isVPN) {  MessageBoxEx.Show(this, "请打开VPN!"); return ; }

            var timeArr =  GetTimeListByDataTable(resultTable);

            //本团行程单数据
            List<Grp_TravelList> listTravel = db.Grp_TravelList.Where(x => x.Diid == diid && x.IsDel == 0).ToList();

            ChildThreadExceptionHandler wt = new
                 ChildThreadExceptionHandler((msg) =>
                 {
                     if (this.InvokeRequired)
                     {
                         this.Invoke(new Action(() =>
                         {
                             MessageBoxEx.Show(this, msg);
                         }));
                     }
                     else
                     {
                         MessageBoxEx.Show(this, msg);
                     }
                 });

            Thread threadExec = new Thread(() =>
            {
                try
                {
                    using (var sw = db.Database.BeginTransaction())
                    {
                        db.Grp_TravelList.RemoveRange(listTravel);
                        db.SaveChanges();

                        var NewListTravel = new List<Grp_TravelList>();
                        var index = 1;
                        var stopCity = string.Empty;

                        foreach (var item in timeArr)
                        {
                            string trip = string.Empty;
                            string weekDay = string.Empty;
                            DateTime time = DateTime.Now;
                            if (DateTime.TryParse(item, out time))
                            {
                                weekDay = weekdays[(int)time.DayOfWeek];
                            }
                            else
                            {
                                weekDay = "日期格式不正确!";
                            }

                            var empty = "【未收入该三字码！请机票同事录入】";
                            var tabSelect = resultTable.Select(string.Format("Day = '{0}'", item));
                            var isMoreTraffic = false;
                            if (tabSelect.Length > 0)
                            {
                                isMoreTraffic = true;
                                var takeOffTime = DateTime.Parse(item);
                                var fallToTime = DateTime.Parse(item);

                                Res_ThreeCode start_Object = null;
                                Res_ThreeCode end_Object = null;

                                bool isTrade = false;

                                foreach (var tabRow in tabSelect)
                                {

                                    takeOffTime = DateTime.Parse(item);
                                    fallToTime = DateTime.Parse(item);

                                    var takeOff = tabRow["StartTime"].ToString();
                                    var fallTo = tabRow["EndTime"].ToString();
                                   
                                    takeOffTime = takeOffTime.AddHours(int.Parse(takeOff.Substring(0, 2)));
                                    takeOffTime = takeOffTime.AddMinutes(int.Parse(takeOff.Substring(2, 2)));
                                   
                                    fallToTime = fallToTime.AddHours(int.Parse(fallTo.Substring(0, 2)));
                                    fallToTime = fallToTime.AddMinutes(int.Parse(fallTo.Substring(2, 2)));

                                    var threeCode = tabRow["Three"].ToString();
                                    var start = threeCode.Substring(0, 3);
                                    var end = threeCode.Substring(3, 3);
                                    stopCity = end;
                                    start_Object = db.Res_ThreeCode.FirstOrDefault(x => x.Three.ToUpper() == start.ToUpper());
                                    end_Object = db.Res_ThreeCode.FirstOrDefault(x => x.Three.ToUpper() == end.ToUpper());
                                    if (start_Object == null)
                                    {
                                        start_Object = new Res_ThreeCode()
                                        {
                                            AirPort = empty,
                                            AirPort_En = empty,
                                            City = empty,
                                            Country = empty,
                                            Four = empty,
                                            Three = empty,
                                        };
                                    }
                                    if (end_Object == null)
                                    {
                                        end_Object = new Res_ThreeCode()
                                        {
                                            AirPort = empty,
                                            AirPort_En = empty,
                                            City = empty,
                                            Country = empty,
                                            Four = empty,
                                            Three = empty,
                                        };
                                    }

                                    //航班号
                                    string flightcode = tabRow["Fliagtcode"].ToString();

                                    if (isTrade)
                                    {
                                        trip += "\r\n";
                                    }

                                    trip += $"{start_Object.City}/{end_Object.City}\r\n";

                                    if (!isTrade)
                                    {
                                        //--提前俩个小时
                                        trip += $"{takeOffTime.AddHours(-2).ToString("HH:mm")} 抵达{start_Object.AirPort}{tabRow["StartBuilding"].ToString().Trim()}航站楼,办理登机手续：\r\n";
                                    }

                                    var aircompany = db.Res_AirCompany.FirstOrDefault(x => x.ShortCode.ToUpper() == flightcode.Substring(0, 2).ToUpper() && x.IsDel == 0);

                                    var hsEmpty = "【此航司" + flightcode.Substring(0, 2).ToUpper() + "未收录，请机票同事录入】";
                                    if (aircompany == null)
                                    {
                                        aircompany = new Res_AirCompany
                                        {
                                            CnName = hsEmpty,
                                            EnName = hsEmpty,
                                            ShortCode = hsEmpty,
                                        };
                                    }

                                    //机型判断
                                    string airModel = tabRow["AirModel"].ToString();
                                    airModel = GetLonger(airModel.Substring(0, 1)) + airModel;

                                    string flightTime = tabRow["FlightTime"].ToString();
                                    if (flightTime.Contains(":"))
                                    {
                                        flightTime = flightTime.Replace(":", "小时");
                                        flightTime += "分钟";
                                    }
                                    if (flightTime.Contains("H"))
                                    {
                                        flightTime = flightTime.Replace("H", "小时");
                                    }
                                    if (flightTime.Contains("M"))
                                    {
                                        flightTime = flightTime.Replace("M", "分钟");
                                    }

                                    trip += $"{takeOffTime.ToString("HH:mm")} 搭乘{aircompany.CnName}{tabRow[0].ToString().Trim()}航班，由{start_Object.City}飞往{end_Object.City}；\r\n       （{start_Object.AirPort}/{end_Object.AirPort}  机型：{airModel}    飞行时间{flightTime}）；\r\n";

                                    isTrade = Convert.ToBoolean(tabRow["isTransitShipment"]);

                                    if (isTrade)
                                    {
                                        trip += $"{fallToTime.ToString("HH:mm")} 抵达{end_Object.AirPort}{tabRow["EndBuilding"].ToString().Trim()}航站楼(中转时间：行李直达)";
                                    }
                                    else
                                    {
                                        trip += $"{fallToTime.ToString("HH:mm")} 抵达{end_Object.AirPort}{tabRow["EndBuilding"].ToString().Trim()}航站楼,办理入境手续，之后前往提取行李\r\n";
                                    }
                                }

                                //var airArrive = fallToTime;      //航班落地时间
                                time = fallToTime.AddHours(1.5); //出机场一个半小时

                                string Time = string.Empty;
                                string Distance = string.Empty;
                                int GetGoogleResult = 0;
                                int GetDistResult = 0;
                                JObject Result = null;
                                try
                                {
                                    Result = GetDirectionByGoogleApi(end_Object.AirPort, end_Object.City + "市区");
                                    Time = Result["routes"][0]["legs"][0]["duration"]["text"].ToString().Replace(" ", "").Trim();
                                    Distance = Result["routes"][0]["legs"][0]["distance"]["text"].ToString().Replace(" ", "").Trim();

                                    if (Time.Contains("hours"))
                                    {
                                        Time = Time.Replace("hours", "小时");
                                    }
                                    if (Time.Contains("hour"))
                                    {
                                        Time = Time.Replace("hour", "小时");
                                    }
                                    Time = Time.Replace("mins", "分钟");

                                    if (Distance.Contains("mi"))
                                    {
                                        var distSp = Regex.Split(Distance, "mi");
                                        if (distSp.Length > 0)
                                        {
                                            Distance = (decimal.Parse(distSp[0]) * conversion).ToString("#0.00") + " 公里";
                                        }
                                    }
                                    if (Distance.Contains("km"))
                                    {
                                        Distance = Distance.Replace("km", "公里");
                                    }

                                    if (Time.Contains("小时"))
                                    {
                                        var xs = Regex.Split(Time, "小时");
                                        var xsValue = int.Parse(xs[0]);
                                        var fz = Regex.Split(xs[1], "分钟");
                                        var fzValue = int.Parse(fz[0]);
                                        GetGoogleResult = xsValue * 60;
                                        GetGoogleResult += fzValue;

                                    }
                                    else if (Time.Contains("分钟"))
                                    {
                                        GetGoogleResult = int.Parse(Regex.Split(Time, "分钟")[0]);
                                    }
                                    if (Distance.Contains("公里"))
                                    {
                                        GetDistResult = (int)decimal.Parse(Regex.Split(Distance, "公里")[0]);
                                    }

                                    if (GetGoogleResult > 0 && GetGoogleResult % 5 != 0)
                                    {
                                        while (GetGoogleResult % 5 != 0)
                                        {
                                            GetGoogleResult++;
                                        }
                                    }
                                    if (GetDistResult % 5 != 0 && GetDistResult > 0)
                                    {
                                        while (GetDistResult % 5 != 0)
                                        {
                                            GetDistResult++;
                                        }
                                    }

                                    int H = (GetGoogleResult / 60);
                                    int m = (GetGoogleResult % 60);
                                    string TimeStr = string.Empty;
                                    if (H != 0)
                                    {
                                        if (H < 10)
                                        {
                                            TimeStr += "0" + H + "小时";
                                        }
                                        else
                                        {
                                            TimeStr += H + "小时";
                                        }
                                    }
                                    if (m < 10)
                                    {
                                        TimeStr += "0" + m + "分钟";
                                    }
                                    else
                                    {
                                        TimeStr += m + "分钟";
                                    }

                                    Time = TimeStr;
                                    Distance = GetDistResult.ToString() + "公里";
                                }
                                catch (Exception ex)
                                {
                                    Time = "未知！";
                                    Distance = "未知！";
                                }

                                trip += $"{time.ToString("HH:mm")} 搭乘专车前往市区（{end_Object.AirPort} - {end_Object.City}市区 路程{Distance}，耗时{Time}）";

                                time = time.AddMinutes(GetGoogleResult);  //到达市区时间

                                if (index == timeArr.Count - 1)
                                {
                                    trip += "圆满结束此次访问之行！";
                                    NewListTravel.Add(new Grp_TravelList
                                    {
                                        CreateTime = DateTime.Now,
                                        CreateUserId = 0,
                                        Diid = diid,
                                        Date = Convert.ToDateTime(item).ToString("M月d日"),
                                        Trip = trip,
                                        WeekDay = weekDay,
                                        Days = index,
                                        Traffic_First = "飞机",
                                        Traffic_Second = "汽车",
                                        Diffgroup = 1,
                                        Issel = 1,
                                        IsDel = 0,
                                    });
                                    continue;
                                }

                                if (time.Day != fallToTime.Day)  //超出一天
                                {
                                    NewListTravel.Add(new Grp_TravelList
                                    {
                                        CreateTime = DateTime.Now,
                                        CreateUserId = 0,
                                        Diid = diid,
                                        Date = Convert.ToDateTime(item).ToString("M月d日"),
                                        Trip = trip,
                                        WeekDay = weekDay,
                                        Days = index,
                                        Traffic_First = "飞机",
                                        Traffic_Second = "汽车",
                                        Diffgroup = 1,
                                        Issel = 1,
                                        IsDel = 0,
                                    });
                                    continue;
                                }

                                if (time.Hour < 9)  // && (airArrive < new DateTime(airArrive.Year,airArrive.Month,airArrive.Day,6, 30, 0))
                                {
                                    trip += @"
09:00 公务活动；
10:30 公务活动；";
                                }
                                else if (time.Hour < 10)
                                {
                                    trip += @"
10:30 公务活动；";
                                }

                                if (time.Hour < 13)
                                {
                                    trip += $"\r\n{time.ToString("HH:mm")} 午餐于当地餐厅;";
                                }

                                if (time < new DateTime(time.Year, time.Month, time.Day, 14, 30, 0))
                                {
                                    trip += @"
14:00 公务活动；
16:00 公务活动；";
                                }
                                else if (time.Hour < 16)
                                {
                                    trip += @"
16:00 公务活动；";
                                }

                                if (time.Hour < 18)
                                {
                                    trip += $"\r\n18:00 晚餐于当地餐厅;";
                                }

                                //time = time.AddHours(1);
                                //trip += $"\r\n{time.ToString("HH:mm")} 搭乘专车前往酒店，抵达后办理入住；";
                            }
                            else
                            {
                                var end_Object = db.Res_ThreeCode.FirstOrDefault(x => x.Three.ToUpper() == stopCity.ToUpper());
                                if (end_Object == null)
                                {
                                    end_Object = new Res_ThreeCode()
                                    {
                                        AirPort = empty,
                                        AirPort_En = empty,
                                        City = empty,
                                        Country = empty,
                                        Four = empty,
                                        Three = empty,
                                    };
                                }
                                trip += $"{end_Object.City}\r\n";
                                trip += @"08:00 早餐于酒店；
09:00 公务活动；
10:30 公务活动；
12:00 午餐于当地餐厅；
14:00 公务活动；
16:00 公务活动；
18:00 晚餐于当地餐厅；
19:00 入住酒店休息；";
                            }

                            string[] traffic = new string[] { "汽车", "飞机" };
                            if (!isMoreTraffic)
                            {
                                traffic = new string[] { "汽车", "" };
                            }

                            NewListTravel.Add(new Grp_TravelList
                            {
                                CreateTime = DateTime.Now,
                                CreateUserId = 0,
                                Diid = diid,
                                Date = Convert.ToDateTime(item).ToString("M月d日"),
                                Trip = trip,
                                WeekDay = weekDay,
                                Days = index,
                                Traffic_First = traffic[0],
                                Traffic_Second = traffic[1],
                                Diffgroup = 1,
                                Issel = 1,
                                IsDel = 0,
                            });

                            index++;
                        }

                        db.Grp_TravelList.AddRange(NewListTravel);
                        db.SaveChanges();

                        sw.Commit();
                        wt.Invoke("数据导入完成！");
                    }
                }
                catch (Exception ex)
                {
                    wt.Invoke("请等待程序执行！");
                }
            });
            threadExec.Start();
        }

        public async void btnImportInfo()
        {
            var diid = -1;
            try
            {
                var dele = (comb_Delegation.SelectedItem as Grp_DelegationInfo);
                diid = dele.Id;
            }
            catch (Exception)
            {
                //lblImport.Text = "请正确的选择团组!";
                //lblImport.ForeColor = Color.Black;
                MessageBoxEx.Show(this, "请正确的选择团组!", "提示");
                return;
            }

            DataTable resultTable = null;
            try
            {
                resultTable = GetTableByBlackCode(diid);
            }
            catch (Exception ex)
            {
                MessageBoxEx.Show(this, "黑屏代码有误!", "提示");
                return;
            }

            if (resultTable.Rows.Count == 0 || string.IsNullOrWhiteSpace(resultTable.Rows[0][1].ToString()))
            {
                string msg = string.Empty;
                if (resultTable.Rows.Count > 0)
                {
                    msg = resultTable.Rows[0]["Error"].ToString();
                }
                MessageBoxEx.Show(this, "黑屏代码有误! " + msg, "提示");
                return;
            }

            var isVPN = await Task.Run(() =>
            {
                var isVpn = GetDirectionByGoogleApi("北京", "上海");
                if (Convert.ToInt32(isVpn["code"]) == -1)
                {
                    return false;
                }
                return true;
            });

            if (!isVPN) { MessageBoxEx.Show(this, "请打开VPN!"); return; }

            var timeArr = GetTimeListByDataTable(resultTable);

            //本团行程单数据
            List<Grp_TravelList> listTravel = db.Grp_TravelList.Where(x => x.Diid == diid && x.IsDel == 0).ToList();

            ChildThreadExceptionHandler wt = new
                 ChildThreadExceptionHandler((msg) =>
                 {
                     if (this.InvokeRequired)
                     {
                         this.Invoke(new Action(() =>
                         {
                             MessageBoxEx.Show(this, msg);
                         }));
                     }
                     else
                     {
                         MessageBoxEx.Show(this, msg);
                     }
                 });

            Thread threadExec = new Thread(() =>
            {
                try
                {
                    using (var sw = db.Database.BeginTransaction())
                    {
                        db.Grp_TravelList.RemoveRange(listTravel);
                        db.SaveChanges();

                        var NewListTravel = new List<Grp_TravelList>();
                        var index = 1;
                        var stopCity = string.Empty;

                        //本团车导地接信息
                        var listctg = db.Grp_CarTouristGuideGroundReservations.Where(x => x.DiId == diid && x.IsDel == 0).ToList();

                        //本团酒店信息
                        var listht = db.Grp_HotelReservations.Where(x => x.DiId == diid && x.IsDel == 0).ToList();

                        //公务出访信息
                        var listgw = db.Res_OfficialActivities.Where(x => x.DiId == diid && x.IsDel == 0).ToList();

                        //邀请公务资料
                        var listgwzl = db.Grp_InvitationOfficialActivities.Where(x => x.DiId == diid && x.IsDel == 0).ToList();

                        Grp_HotelReservations lastHotel = null;

                        foreach (var item in timeArr)
                        {
                            string trip = string.Empty;
                            string weekDay = string.Empty;
                            DateTime time = DateTime.Now;
                            if (DateTime.TryParse(item, out time))
                            {
                                weekDay = weekdays[(int)time.DayOfWeek];
                            }
                            else
                            {
                                weekDay = "日期格式不正确!";
                            }

                            var empty = "【未收入该三字码！请机票同事录入】";
                            var tabSelect = resultTable.Select(string.Format("Day = '{0}'", item));
                            var isMoreTraffic = false;
                            if (tabSelect.Length > 0)
                            {
                                isMoreTraffic = true;
                                var takeOffTime = DateTime.Parse(item);
                                var fallToTime = DateTime.Parse(item);

                                Res_ThreeCode start_Object = null;
                                Res_ThreeCode end_Object = null;

                                bool isTrade = false;

                                foreach (var tabRow in tabSelect)
                                {

                                    takeOffTime = DateTime.Parse(item);
                                    fallToTime = DateTime.Parse(item);

                                    var takeOff = tabRow["StartTime"].ToString();
                                    var fallTo = tabRow["EndTime"].ToString();

                                    takeOffTime = takeOffTime.AddHours(int.Parse(takeOff.Substring(0, 2)));
                                    takeOffTime = takeOffTime.AddMinutes(int.Parse(takeOff.Substring(2, 2)));

                                    fallToTime = fallToTime.AddHours(int.Parse(fallTo.Substring(0, 2)));
                                    fallToTime = fallToTime.AddMinutes(int.Parse(fallTo.Substring(2, 2)));

                                    var threeCode = tabRow["Three"].ToString();
                                    var start = threeCode.Substring(0, 3);
                                    var end = threeCode.Substring(3, 3);
                                    stopCity = end;
                                    start_Object = db.Res_ThreeCode.FirstOrDefault(x => x.Three.ToUpper() == start.ToUpper());
                                    end_Object = db.Res_ThreeCode.FirstOrDefault(x => x.Three.ToUpper() == end.ToUpper());
                                    if (start_Object == null)
                                    {
                                        start_Object = new Res_ThreeCode()
                                        {
                                            AirPort = empty,
                                            AirPort_En = empty,
                                            City = empty,
                                            Country = empty,
                                            Four = empty,
                                            Three = empty,
                                        };
                                    }
                                    if (end_Object == null)
                                    {
                                        end_Object = new Res_ThreeCode()
                                        {
                                            AirPort = empty,
                                            AirPort_En = empty,
                                            City = empty,
                                            Country = empty,
                                            Four = empty,
                                            Three = empty,
                                        };
                                    }

                                    //航班号
                                    string flightcode = tabRow["Fliagtcode"].ToString();

                                    if (isTrade)
                                    {
                                        trip += "\r\n";
                                    }

                                    trip += $"{start_Object.City}/{end_Object.City}\r\n";

                                    //--提前俩个小时
                                    if (!isTrade)
                                    {
                                        //--提前俩个小时
                                        trip += $"{takeOffTime.AddHours(-2).ToString("HH:mm")} 抵达{start_Object.AirPort}{tabRow["StartBuilding"].ToString().Trim()}航站楼,办理登机手续：\r\n";
                                    }

                                    var aircompany = db.Res_AirCompany.FirstOrDefault(x => x.ShortCode.ToUpper() == flightcode.Substring(0, 2).ToUpper() && x.IsDel == 0);

                                    var hsEmpty = "【此航司" + flightcode.Substring(0, 2).ToUpper() + "未收录，请机票同事录入】";
                                    if (aircompany == null)
                                    {
                                        aircompany = new Res_AirCompany
                                        {
                                            CnName = hsEmpty,
                                            EnName = hsEmpty,
                                            ShortCode = hsEmpty,
                                        };
                                    }

                                    //机型判断
                                    string airModel = tabRow["AirModel"].ToString();
                                    airModel = GetLonger(airModel.Substring(0, 1)) + airModel;

                                    string flightTime = tabRow["FlightTime"].ToString();
                                    if (flightTime.Contains(":"))
                                    {
                                        flightTime = flightTime.Replace(":", "小时");
                                        flightTime += "分钟";
                                    }
                                    if (flightTime.Contains("H"))
                                    {
                                        flightTime = flightTime.Replace("H", "小时");
                                    }
                                    if (flightTime.Contains("M"))
                                    {
                                        flightTime = flightTime.Replace("M", "分钟");
                                    }

                                    trip += $"{takeOffTime.ToString("HH:mm")} 搭乘{aircompany.CnName}{tabRow[0].ToString().Trim()}航班，由{start_Object.City}飞往{end_Object.City}；\r\n       （{start_Object.AirPort}/{end_Object.AirPort}  机型：{airModel}    飞行时间{flightTime}）；\r\n";

                                    isTrade = Convert.ToBoolean(tabRow["isTransitShipment"]);

                                    if (isTrade)
                                    {
                                        trip += $"{fallToTime.ToString("HH:mm")} 抵达{end_Object.AirPort}{tabRow["EndBuilding"].ToString().Trim()}航站楼(中转时间：行李直达)";
                                    }
                                    else
                                    {
                                        trip += $"{fallToTime.ToString("HH:mm")} 抵达{end_Object.AirPort}{tabRow["EndBuilding"].ToString().Trim()}航站楼,办理入境手续，之后前往提取行李\r\n";
                                    }
                                }

                                //var airArrive = fallToTime;      //航班落地时间
                                time = fallToTime.AddHours(1.5); //出机场一个半小时

                                string Time = string.Empty;
                                string Distance = string.Empty;
                                int GetGoogleResult = 0;
                                int GetDistResult = 0;
                                JObject Result = null;
                                try
                                {
                                    Result = GetDirectionByGoogleApi(end_Object.AirPort, end_Object.City + "市区");
                                    Time = Result["routes"][0]["legs"][0]["duration"]["text"].ToString().Replace(" ", "").Trim();
                                    Distance = Result["routes"][0]["legs"][0]["distance"]["text"].ToString().Replace(" ", "").Trim();

                                    if (Time.Contains("hours"))
                                    {
                                        Time = Time.Replace("hours", "小时");
                                    }
                                    if (Time.Contains("hour"))
                                    {
                                        Time = Time.Replace("hour", "小时");
                                    }
                                    Time = Time.Replace("mins", "分钟");

                                    if (Distance.Contains("mi"))
                                    {
                                        var distSp = Regex.Split(Distance, "mi");
                                        if (distSp.Length > 0)
                                        {
                                            Distance = (decimal.Parse(distSp[0]) * conversion).ToString("#0.00") + " 公里";
                                        }
                                    }
                                    if (Distance.Contains("km"))
                                    {
                                        Distance = Distance.Replace("km", "公里");
                                    }

                                    if (Time.Contains("小时"))
                                    {
                                        var xs = Regex.Split(Time, "小时");
                                        var xsValue = int.Parse(xs[0]);
                                        var fz = Regex.Split(xs[1], "分钟");
                                        var fzValue = int.Parse(fz[0]);
                                        GetGoogleResult = xsValue * 60;
                                        GetGoogleResult += fzValue;

                                    }
                                    else if (Time.Contains("分钟"))
                                    {
                                        GetGoogleResult = int.Parse(Regex.Split(Time, "分钟")[0]);
                                    }
                                    if (Distance.Contains("公里"))
                                    {
                                        GetDistResult = (int)decimal.Parse(Regex.Split(Distance, "公里")[0]);
                                    }

                                    if (GetGoogleResult > 0 && GetGoogleResult % 5 != 0)
                                    {
                                        while (GetGoogleResult % 5 != 0)
                                        {
                                            GetGoogleResult++;
                                        }
                                    }
                                    if (GetDistResult % 5 != 0 && GetDistResult > 0)
                                    {
                                        while (GetDistResult % 5 != 0)
                                        {
                                            GetDistResult++;
                                        }
                                    }

                                    int H = (GetGoogleResult / 60);
                                    int m = (GetGoogleResult % 60);
                                    string TimeStr = string.Empty;
                                    if (H != 0)
                                    {
                                        if (H < 10)
                                        {
                                            TimeStr += "0" + H + "小时";
                                        }
                                        else
                                        {
                                            TimeStr += H + "小时";
                                        }
                                    }
                                    if (m < 10)
                                    {
                                        TimeStr += "0" + m + "分钟";
                                    }
                                    else
                                    {
                                        TimeStr += m + "分钟";
                                    }

                                    Time = TimeStr;
                                    Distance = GetDistResult.ToString() + "公里";
                                }
                                catch (Exception ex)
                                {
                                    Time = "未知！";
                                    Distance = "未知！";
                                }

                                trip += $"{time.ToString("HH:mm")} 搭乘专车前往市区（{end_Object.AirPort} - {end_Object.City}市区 路程{Distance}，耗时{Time}）";

                                time = time.AddMinutes(GetGoogleResult);  //到达市区时间

                                if (index == timeArr.Count - 1)
                                {
                                    trip += "圆满结束此次访问之行！";
                                    NewListTravel.Add(new Grp_TravelList
                                    {
                                        CreateTime = DateTime.Now,
                                        CreateUserId = 0,
                                        Diid = diid,
                                        Date = Convert.ToDateTime(item).ToString("M月d日"),
                                        Trip = trip,
                                        WeekDay = weekDay,
                                        Days = index,
                                        Traffic_First = "飞机",
                                        Traffic_Second = "汽车",
                                        Diffgroup = 1,
                                        Issel = 1,
                                        IsDel = 0,
                                    });
                                    continue;
                                }

                                if (time.Day != fallToTime.Day)  //超出一天
                                {
                                    NewListTravel.Add(new Grp_TravelList
                                    {
                                        CreateTime = DateTime.Now,
                                        CreateUserId = 0,
                                        Diid = diid,
                                        Date = Convert.ToDateTime(item).ToString("M月d日"),
                                        Trip = trip,
                                        WeekDay = weekDay,
                                        Days = index,
                                        Traffic_First = "飞机",
                                        Traffic_Second = "汽车",
                                        Diffgroup = 1,
                                        Issel = 1,
                                        IsDel = 0,
                                    });
                                    continue;
                                }

                                if (time.Hour < 9)  // && (airArrive < new DateTime(airArrive.Year,airArrive.Month,airArrive.Day,6, 30, 0))
                                {
                                    var nineTime =  GetTimeGwInfo(ref listgw, new DateTime(time.Year,time.Month,time.Day,09,00,00),null);
                                    var halfPasTen = GetTimeGwInfo(ref listgw, new DateTime(time.Year, time.Month, time.Day, 10, 30, 00), null);
                                    trip += @"
09:00 公务活动；
10:30 公务活动；";
                                    if (!string.IsNullOrWhiteSpace(nineTime))
                                    {
                                        trip = trip.Replace("09:00 公务活动", nineTime);
                                    }
                                    if (!string.IsNullOrWhiteSpace(halfPasTen))
                                    {
                                        trip = trip.Replace("10:30 公务活动", halfPasTen);
                                    }

                                }
                                else if (time.Hour < 10)
                                {
                                    var halfPasTen = GetTimeGwInfo(ref listgw, new DateTime(time.Year, time.Month, time.Day, 10, 30, 00), null);
                                    trip += @"
10:30 公务活动；";
                                    if (!string.IsNullOrWhiteSpace(halfPasTen))
                                    {
                                        trip = trip.Replace("10:30 公务活动", halfPasTen);
                                    }
                                }

                                if (time.Hour < 13)
                                {
                                    trip += $"\r\n{time.ToString("HH:mm")} 午餐于当地餐厅;";
                                }

                                if (time < new DateTime(time.Year, time.Month, time.Day, 14, 30, 0))
                                {
                                    var twoPointsPM = GetTimeGwInfo(ref listgw, new DateTime(time.Year, time.Month, time.Day, 14, 00, 00), null);
                                    var fourPointsPM = GetTimeGwInfo(ref listgw, new DateTime(time.Year, time.Month, time.Day, 16, 00, 00), null);
                                    trip += @"
14:00 公务活动；
16:00 公务活动；";
                                    if (!string.IsNullOrWhiteSpace(twoPointsPM))
                                    {
                                        trip = trip.Replace("14:00 公务活动", twoPointsPM);
                                    }
                                    if (!string.IsNullOrWhiteSpace(fourPointsPM))
                                    {
                                        trip = trip.Replace("16:00 公务活动", fourPointsPM);
                                    }
                                }
                                else if (time.Hour < 16)
                                {
                                    var fourPointsPM = GetTimeGwInfo(ref listgw, new DateTime(time.Year, time.Month, time.Day, 16, 00, 00), null);
                                    trip += @"
16:00 公务活动；";
                                    if (!string.IsNullOrWhiteSpace(fourPointsPM))
                                    {
                                        trip = trip.Replace("16:00 公务活动", fourPointsPM);
                                    }
                                }

                                if (time.Hour < 18)
                                {
                                    trip += $"\r\n18:00 晚餐于当地餐厅;";
                                }

                                //time = time.AddHours(1);
                                //trip += $"\r\n{time.ToString("HH:mm")} 搭乘专车前往酒店，抵达后办理入住；";
                            }
                            else
                            {
                                var end_Object = db.Res_ThreeCode.FirstOrDefault(x => x.Three.ToUpper() == stopCity.ToUpper());
                                if (end_Object == null)
                                {
                                    end_Object = new Res_ThreeCode()
                                    {
                                        AirPort = empty,
                                        AirPort_En = empty,
                                        City = empty,
                                        Country = empty,
                                        Four = empty,
                                        Three = empty,
                                    };
                                }
                                trip += $"{end_Object.City}\r\n";
                                trip += $@"08:00 早餐于酒店；{(lastHotel != null? $"（酒店名称：{lastHotel.HotelName}）":"")}
09:00 公务活动；
10:30 公务活动；
12:00 午餐于当地餐厅；
14:00 公务活动；
16:00 公务活动；
18:00 晚餐于当地餐厅；
19:00 入住酒店休息；";

                                var nineTime = GetTimeGwInfo(ref listgw, new DateTime(time.Year, time.Month, time.Day, 09, 00, 00), end_Object.City);
                                var halfPasTen = GetTimeGwInfo(ref listgw, new DateTime(time.Year, time.Month, time.Day, 10, 30, 00), end_Object.City);
                                var twoPointsPM = GetTimeGwInfo(ref listgw, new DateTime(time.Year, time.Month, time.Day, 14, 00, 00), end_Object.City);
                                var fourPointsPM = GetTimeGwInfo(ref listgw, new DateTime(time.Year, time.Month, time.Day, 16, 00, 00), end_Object.City);
                                if (!string.IsNullOrWhiteSpace(nineTime))
                                {
                                    trip = trip.Replace("09:00 公务活动", nineTime);
                                }
                                if (!string.IsNullOrWhiteSpace(halfPasTen))
                                {
                                    trip = trip.Replace("10:30 公务活动", halfPasTen);
                                }
                                if (!string.IsNullOrWhiteSpace(twoPointsPM))
                                {
                                    trip = trip.Replace("14:00 公务活动", twoPointsPM);
                                }
                                if (!string.IsNullOrWhiteSpace(fourPointsPM))
                                {
                                    trip = trip.Replace("16:00 公务活动", fourPointsPM);
                                }
                            }

                            string[] traffic = new string[] { "汽车", "飞机" };
                            if (!isMoreTraffic)
                            {
                                traffic = new string[] { "汽车", "" };
                            }

                            var htTime = DateTime.Now;
                            var htTimeOut = DateTime.Now;
                            foreach (var h in listht)
                            {
                                if (DateTime.TryParse(h.CheckInDate, out htTime) && DateTime.TryParse(h.CheckOutDate, out htTimeOut))
                                {
                                    if (htTime <= Convert.ToDateTime(item)
                                     && htTimeOut > Convert.ToDateTime(item))
                                    {
                                        lastHotel = h;

                                        #region 酒店详细信息
                                        //trip += "\r\n00:00 乘车前往酒店";
                                        //trip += "\r\n00:00 抵达酒店办理入住手续";

                                        //trip = trip + "\r\n"
                                        //+ "          酒店名称：" + h.HotelName + "\r\n"
                                        //+ "          酒店地址：" + h.HotelAddress + "\r\n"
                                        //+ "          酒店电话：" + h.HotelTel + "      酒店传真：" + (string.IsNullOrWhiteSpace(h.HotelFax) ? "-" : h.HotelFax);
                                        #endregion
                                    }
                                }
                            }

                            NewListTravel.Add(new Grp_TravelList
                            {
                                CreateTime = DateTime.Now,
                                CreateUserId = 0,
                                Diid = diid,
                                Date = Convert.ToDateTime(item).ToString("M月d日"),
                                Trip = trip,
                                WeekDay = weekDay,
                                Days = index,
                                Traffic_First = traffic[0],
                                Traffic_Second = traffic[1],
                                Diffgroup = 1,
                                Issel = 1,
                                IsDel = 0,
                            });

                            index++;
                        }

                        NewListTravel.ForEach(x =>
                        {
                            x.Trip = x.Trip.Replace("\r\n14:00 公务活动；", string.Empty)
                                           .Replace("\r\n16:00 公务活动；", string.Empty)
                                           .Replace("\r\n09:00 公务活动；", string.Empty)
                                           .Replace("\r\n10:30 公务活动；", string.Empty);
                        });

                        db.Grp_TravelList.AddRange(NewListTravel);
                        db.SaveChanges();

                        sw.Commit();
                        wt.Invoke("数据导入完成！");
                    }
                }
                catch (Exception ex)
                {
                    wt.Invoke("请等待程序执行！");
                }
            });
            threadExec.Start();
        }

        private delegate void ChildThreadExceptionHandler(string message);

        private void btnOutput_Click(object sender, EventArgs e)
        {
            var diid = -1;
            try
            {
                var dele = (comb_Delegation.SelectedItem as Grp_DelegationInfo);
                diid = dele.Id;
            }
            catch (Exception)
            {
                //lblImport.Text = "请正确的选择团组!";
                //lblImport.ForeColor = Color.Black;
                MessageBoxEx.Show(this, "请正确的选择团组!", "提示");
                return;
            }

            Grp_DelegationInfo di = db.Grp_DelegationInfo.FirstOrDefault(x => x.Id == diid && x.IsDel == 0);

            if (di == null)
            {
                MessageBoxEx.Show(this,"请选择正确的团组!");
                return;
            }

            var _travelList = db.Grp_TravelList.Where(x=>x.IsDel == 0 && x.Diid == diid).ToList();

            var city =  GetAllCity(diid);

            //创建数据源Table
            DataTable dtSource = new DataTable();
            dtSource.Columns.Add("Days", typeof(string));
            dtSource.Columns.Add("Date", typeof(string));
            dtSource.Columns.Add("Week", typeof(string));
            dtSource.Columns.Add("Traffic", typeof(string));
            dtSource.Columns.Add("Trip", typeof(string));

            //获取数据,放到datatable
            foreach (var item in _travelList)
            {
                DataRow dr = dtSource.NewRow();
                dr["Days"] = item.Days;
                dr["Date"] = item.Date;
                dr["Week"] = item.WeekDay;
                dr["Traffic"] = item.Traffic_First
                                + "\r\n"
                                + item.Traffic_Second;
                dr["Trip"] = item.Trip;
                dtSource.Rows.Add(dr);
            }

            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add("Dele", di.TeamName.ToString() + GetNum(di.VisitDays.ToString()));
            dic.Add("City", city);
            dic.Add("Days", di.VisitDays.ToString());
            dic.Add("DeleCode", di.TourCode);
            dic.Add("Pnum", di.VisitPNumber.ToString());
            dic.Add("Pnum2", di.VisitPNumber.ToString());

            //模板路径
            string tempPath = @"C:\日行程3.docx";
            //载入模板
            Document doc = null;
            DocumentBuilder builder = null;
            try
            {
                //载入模板
                doc = new Document(tempPath);
                builder = new DocumentBuilder(doc);
            }
            catch (Exception)
            {
                //载入模板
                doc = new Document(@"D:\日行程3.docx");
                builder = new DocumentBuilder(doc);
            }

            foreach (var key in dic.Keys)
            {
                Bookmark bookmark = doc.Range.Bookmarks[key];
                if (bookmark != null)
                {
                    builder.MoveToBookmark(key);
                    builder.Write(dic[key]);
                }
            }
            
            //获取word里所有表格
            NodeCollection allTables = doc.GetChildNodes(NodeType.Table, true);
            //获取所填表格的序数
            Aspose.Words.Tables.Table tableOne = allTables[0] as Aspose.Words.Tables.Table;

            try
            {
                //循环赋值
                for (int i = 0; i < dtSource.Rows.Count; i++)
                {
                    builder.MoveToCell(0, i + 1, 0, 0);
                    builder.Write(dtSource.Rows[i]["Days"].ToString());

                    builder.MoveToCell(0, i + 1, 1, 0);
                    builder.Write(dtSource.Rows[i]["Date"].ToString() + "\r\n" + dtSource.Rows[i]["Week"].ToString());

                    builder.MoveToCell(0, i + 1, 2, 0);
                    builder.Write(dtSource.Rows[i]["Traffic"].ToString());

                    var trip = dtSource.Rows[i]["Trip"].ToString();
                    builder.MoveToCell(0, i + 1, 3, 0);
                    builder.Write(trip);

                    var cell = (Aspose.Words.Tables.Cell)doc.GetChild(NodeType.Cell, ((i + 2) * 4) - 1, true);
                    var paragraphs = cell.GetChildNodes(NodeType.Paragraph, true);

                    // 获取特定索引的段落
                    Paragraph paragraph = (Paragraph)paragraphs[0];
                    Run run = paragraph.Runs[0];
                    Aspose.Words.Font font = run.Font;
                    font.Name = "黑体";

                    //设置双休红色
                    cell = (Aspose.Words.Tables.Cell)doc.GetChild(NodeType.Cell, ((i + 2) * 4) - 3, true);
                    paragraphs = cell.GetChildNodes(NodeType.Paragraph, true);
                    paragraph = (Paragraph)paragraphs[1];
                    if (paragraph.GetText().Contains("星期六") || paragraph.GetText().Contains("星期日"))
                    {
                        run = paragraph.Runs[0];
                        font = run.Font;
                        font.Color = Color.Red;
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBoxEx.Show(this, "错误!" + ex.Message);
                return ;
            }

            //删除多余行
            while (tableOne.Rows.Count > 1 + dtSource.Rows.Count)
            {
                tableOne.Rows.RemoveAt(1 + dtSource.Rows.Count);//(1+dtSource.Rows.Count + 1)-1
            }

            string savePath = "C:\\OP行程单\\";
            if (!Directory.Exists(savePath))
            {
                try
                {
                    Directory.CreateDirectory(savePath);
                }
                catch
                {

                }
            }
            string strFileName = savePath + di.TeamName + "出访日程.docx";

            try
            {
                doc.Save(strFileName, Aspose.Words.SaveFormat.Doc);
                MessageBoxEx.Show(this, "文件导出完成！");
                System.Diagnostics.Process.Start("Explorer.exe", "/select," + strFileName);
            }
            catch (Exception ex)
            {
                MessageBoxEx.Show(this, "请关闭打开的文件后重试！");
            }
        }

        private string GetAllCity(int diid)
        {
            //城市缓存
            string CityStr = string.Empty;

            DataTable dtBlack = null;

            try
            {
                dtBlack = GetTableByBlackCode(diid);
            }
            catch (Exception ex)
            {
                MessageBoxEx.Show(this, "黑屏代码有误!", "提示");
                return CityStr;
            }

            if (dtBlack.Rows.Count != 0 && (!string.IsNullOrWhiteSpace(dtBlack.Rows[0][0].ToString())))
            {
                var cityThree = new List<string>();
                
                foreach (DataRow item in dtBlack.Rows)
                {
                    var three = item["Three"].ToString();
                    if (three.Length == 6)
                    {
                        var start = three.Substring(0, 3);
                        var end = three.Substring(3,3);
                        var lastEnd = string.Empty;
                        if (cityThree.Count != 0)
                        {
                            lastEnd = cityThree[cityThree.Count-1];
                            if (lastEnd != start)
                            {
                                cityThree.Add(start);
                                cityThree.Add(end);
                            }
                            else
                            {
                                cityThree.Add(end);
                            }
                        }
                        else
                        {
                            cityThree.Add(start);
                            cityThree.Add(end);
                        }
                    }
                }
              
                string sql = "select *  from Res_ThreeCode a where a.isdel = 0 ";
                if (cityThree .Count == 0)
                {
                    sql += "and a.id in (0)";
                }
                else
                {
                    sql += $"and a.Three in ('{ string.Join("','",cityThree)}')";
                }

                var resultArr =  db.Res_ThreeCode.SqlQuery(sql).ToList();

                cityThree.ForEach(x =>
                {
                    var find = resultArr.Find(p => p.Three.ToUpper() == x.ToUpper());
                    CityStr += find == null ? $"【{x}三字码未收录】/" : find.City + "/";
                });

                CityStr = CityStr.TrimEnd('/');
            }

            return CityStr;
        }

        /// <summary>
        ///根据机票黑屏代码整理DataTable
        /// </summary>
        /// <param name="diid"></param>
        /// <returns></returns>
        public DataTable GetTableByBlackCode(int diid)
        {
            //黑屏代码信息
            List<Air_TicketBlackCode> listcode = db.Air_TicketBlackCode.Where(x => x.DiId == diid && x.IsDel == 0).ToList();

            //测试数据为序号,航班号,起飞日期,三字码,起飞时刻,到达时刻,出发航站楼,到达航站楼,机型,飞行时间
            //1.3U8391   TU17NOV CTUCAI  0220 0715   T1 T2  330  10H55M
            DataTable dt = new DataTable();
            dt.Columns.Add("Fliagtcode", typeof(string)); //航班号
            dt.Columns.Add("Date", typeof(string));//起飞日期
            dt.Columns.Add("Three", typeof(string));//三字码
            dt.Columns.Add("StartTime", typeof(string));//起飞时刻
            dt.Columns.Add("EndTime", typeof(string));//到达时刻
            dt.Columns.Add("StartBuilding", typeof(string));//出发航站楼
            dt.Columns.Add("EndBuilding", typeof(string));//到达航站楼
            dt.Columns.Add("AirModel", typeof(string)); //机型
            dt.Columns.Add("FlightTime", typeof(string));//飞行时间
            dt.Columns.Add("Day", typeof(string));//整理的起飞日期;作为排序依据 
            dt.Columns.Add("ArrivedDate", typeof(string));//整理的到达日期
            dt.Columns.Add("Error", typeof(string));//整理的到达日期
            dt.Columns.Add("Sign", typeof(string));//标识：0表示为原生黑屏代码、1表示“+1”新增的黑屏代码

            //添加转机标识
            dt.Columns.Add("isTransitShipment", typeof(bool));

            //判断是否录入黑屏代码
            if (listcode.Count() == 0 || listcode == null)
            {
                dt.Rows.Add(null, null, null, null, null, null, null, null, null, null, null, "黑屏代码未录入!", null , false);
            }
            else
            {
                //读取单段黑屏代码
                for (int i = 0; i < listcode.Count; i++)
                {
                    //去除序号
                    string[] CodeList = Regex.Split(listcode[i].BlackCode, "\\d+\\.", RegexOptions.IgnoreCase);
                    //去除多余空格，方法一Linq扩展方法
                    CodeList = CodeList.Where(str =>!string.IsNullOrWhiteSpace(str)).ToArray();
                    
                    //读取单条黑屏代码
                    for (int j = 0; j < CodeList.Count(); j++)
                    {
                        //去除多余空格，方法二使用Split()方法进行分割,分割有一个选项是RemoveEmptyEntries
                        CodeList[j] = CodeList[j].Replace("\r\n", string.Empty).Replace("\\r\\n", string.Empty).TrimStart().TrimEnd();
                        string[] Info = CodeList[j].Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);

                        //去除多余空格
                        Info = Info.Where(str => !string.IsNullOrWhiteSpace(str)).ToArray();

                        //判断黑屏代码是否正确拆分; 理应拆成9段
                        if (Info.TakeWhile((x) =>
                        {
                            return !excludeArr.Contains(x);
                        }).Count() != 9)
                        {
                            dt.Rows.Add(null, null, null, null, null, null, null, null, null, null, null, "本团组第" + (i + 1) + "段黑屏代码中第" + (j + 1) + " 条有误,请联系机票同事核对", null, false);
                            //MessageBoxEx.Show("第" + (i + 1) + "段黑屏代码中第" + (j + 1) + " 条有误, 请联系机票同事核对");
                            return dt;
                        }
                        else
                        {
                            try
                            {
                                var monthEn = Info[1].Substring(4, 3);

                                //月                  
                                int month = Convert.ToInt32(GetLonger(monthEn));
                                //日
                                int day = Convert.ToInt32(Info[1].Substring(2, 2));

                                var time = new DateTime(DateTime.Now.Year, month, day); //

                                var isExist = Info.Contains("[中转]");
                                if (!isExist)
                                {
                                    isExist = Info.Contains("[转机]");
                                }

                                //dt.Columns.Add("Fliagtcode", typeof(string)); //航班号
                                //dt.Columns.Add("Date", typeof(string));//起飞日期
                                //dt.Columns.Add("Three", typeof(string));//三字码
                                //dt.Columns.Add("StartTime", typeof(string));//起飞时刻
                                //dt.Columns.Add("EndTime", typeof(string));//到达时刻
                                //dt.Columns.Add("StartBuilding", typeof(string));//出发航站楼
                                //dt.Columns.Add("EndBuilding", typeof(string));//到达航站楼
                                //dt.Columns.Add("AirModel", typeof(string)); //机型
                                //dt.Columns.Add("FlightTime", typeof(string));//飞行时间
                                //dt.Columns.Add("Day", typeof(string));//整理的起飞日期;作为排序依据 
                                //dt.Columns.Add("ArrivedDate", typeof(string));//整理的到达日期
                                //dt.Columns.Add("Error", typeof(string));//整理的到达日期
                                //dt.Columns.Add("Sign", typeof(string));//标识：0表示为原生黑屏代码、1表示“+1”新增的黑屏代码
                                dt.Rows.Add(Info[0],
                                       Info[1],
                                       Info[2],
                                       Info[3],
                                       Info[4],
                                       Info[5],
                                       Info[6],
                                       Info[7],
                                       Info[8],
                                       time.ToString("yyyy-MM-dd"),
                                       time.ToString("yyyy-MM-dd"),
                                       "",
                                       "0",
                                       isExist
                                       );

                            }
                            catch (Exception ex)
                            {
                                string exstr = ex.Message.ToString();
                            }
                        }

                    }

                    //排序
                    dt.DefaultView.Sort = "Day asc";
                    dt = dt.DefaultView.ToTable();
                }
            }

            return dt;
        }

        public string GetLonger(string temp)
        {
            string str = "";
            switch (temp.ToUpper())
            {
                case "美元":
                    str = "USD";
                    break;
                case "日元":
                    str = "JPY";
                    break;
                case "英镑":
                    str = "GBP";
                    break;
                case "欧元":
                    str = "EUR";
                    break;
                case "港币":
                    str = "HKD";
                    break;
                case "MO":
                    str = "星期一";
                    break;
                case "TU":
                    str = "星期二";
                    break;
                case "WE":
                    str = "星期三";
                    break;
                case "TH":
                    str = "星期四";
                    break;
                case "FR":
                    str = "星期五";
                    break;
                case "SA":
                    str = "星期六";
                    break;
                case "SU":
                    str = "星期天";
                    break;
                case "JAN":
                    str = "01";
                    break;
                case "FEB":
                    str = "02";
                    break;
                case "MAR":
                    str = "03";
                    break;
                case "APR":
                    str = "04";
                    break;
                case "MAY":
                    str = "05";
                    break;
                case "JUN":
                    str = "06";
                    break;
                case "JUL":
                    str = "07";
                    break;
                case "AUG":
                    str = "08";
                    break;
                case "SEP":
                    str = "09";
                    break;
                case "OCT":
                    str = "10";
                    break;
                case "NOV":
                    str = "11";
                    break;
                case "DEC":
                    str = "12";
                    break;
                case "MONDAY":
                    str = "星期一";
                    break;
                case "TUESDAY":
                    str = "星期二";
                    break;
                case "WEDNESDAY":
                    str = "星期三";
                    break;
                case "THURSDAY":
                    str = "星期四";
                    break;
                case "FRIDAY":
                    str = "星期五";
                    break;
                case "SATURDAY":
                    str = "星期六";
                    break;
                case "SUNDAY":
                    str = "星期日";
                    break;
                case "01":
                    str = "JAN";
                    break;
                case "02":
                    str = "FEB";
                    break;
                case "03":
                    str = "MAR";
                    break;
                case "04":
                    str = "APR";
                    break;
                case "05":
                    str = "MAY";
                    break;
                case "06":
                    str = "JUN";
                    break;
                case "07":
                    str = "JUL";
                    break;
                case "08":
                    str = "AUG";
                    break;
                case "09":
                    str = "SEP";
                    break;
                case "10":
                    str = "OCT";
                    break;
                case "11":
                    str = "NOV";
                    break;
                case "12":
                    str = "DEC";
                    break;
                case "2":
                    str = "空客A";
                    break;
                case "3":
                    str = "空客A";
                    break;
                case "7":
                    str = "波音";
                    break;
            }

            return str;
        }

        public List<string> GetTimeListByDataTable(DataTable dt)
        {
            DateTime datestart = Convert.ToDateTime(dt.Rows[0]["Day"].ToString());
            DateTime dateend = Convert.ToDateTime(dt.Rows[dt.Rows.Count - 1]["ArrivedDate"].ToString());

            List<string> timeList = new List<string>();
            while (datestart <= dateend)
            {
                timeList.Add(datestart.ToString("yyyy-MM-dd"));
                datestart = datestart.AddDays(1);
            }
            return timeList;
        }

        public JObject GetDirectionByGoogleApi(string Origin, string Destination)
        {

            string appkey = "AIzaSyBdLf8u8DinXQWVPLEkdrxOJpClXSqEnho";
            string url = "https://maps.googleapis.com/maps/api/directions/json?origin=" + Origin
                       + "&destination=" + Destination + "&key=" + appkey;

            JObject result = null;
            try
            {
                #region 利用HttpWebRequest访问API
                ////创建Web访问对象
                //HttpWebRequest req = HttpWebRequest.Create(url) as HttpWebRequest;
                //req.Method = "GET";
                //req.ContentType = "application/json";
                ////创建web响应对象
                //HttpWebResponse resp = (HttpWebResponse)req.GetResponse();
                ////通过响应内容流创建StreamReader对象，因为StreamReader更高级更快
                //StreamReader reader = new StreamReader(resp.GetResponseStream(), Encoding.UTF8);
                ////如果有编码问题就用这个方法
                ////string returnXml = HttpUtility.UrlDecode(reader.ReadToEnd());
                ////利用StreamReader就可以从响应内容从头读到尾
                //string returnJson = reader.ReadToEnd();

                ////反序列化
                //result = JsonConvert.DeserializeObject(returnJson) as JObject;
                #endregion

                #region 利用HttpClient访问API
                HttpClient myHttpClient = new HttpClient();
                myHttpClient.Timeout = new TimeSpan(0,0,3);
                HttpResponseMessage response = myHttpClient.GetAsync(url).Result;
                //var content = response.Content.ReadAsAsync<object>().Result;
                var content = response.Content.ReadAsStringAsync().Result;

                result = JObject.Parse(content);
                #endregion

                return result;
            }
            catch (Exception e)
            {
                result = new JObject();
                result.Add("code",-1);
                result.Add("msg",e.Message);
                return result;
            }
        }

        bool IsShengGenGuo(string country,string toCountry)
        {
            bool isTrue  = false;
            string[] countries = {
                "德国", "奥地利", "比利时", "捷克共和国", "丹麦", "斯洛伐克", "斯洛文尼亚", "西班牙",
                "爱沙尼亚", "芬兰", "法国", "希腊", "匈牙利", "冰岛", "意大利", "拉脱维亚", "列支敦士登",
                "立陶宛", "卢森堡", "马耳他", "荷兰", "挪威", "波兰", "葡萄牙", "瑞典", "瑞士"
            };

            if (countries.Contains(country)  && countries.Contains(toCountry))
            {
                isTrue = true;
            }

            return isTrue;
        }

        private void btnOutputInfo_Click(object sender, EventArgs e)
        {
            btnImportInfo();
        }

        /// <summary>
        /// 根据大写数字返回小写数字或根据小写数字返回大写数字
        /// </summary>
        /// <param name="num"></param>
        /// <returns></returns>
        public string GetNum(string num)
        {

            string str = "";
            switch (num)
            {
                case "1":
                    str = "一";
                    break;
                case "2":
                    str = "二";
                    break;
                case "3":
                    str = "三";
                    break;
                case "4":
                    str = "四";
                    break;
                case "5":
                    str = "五";
                    break;
                case "6":
                    str = "六";
                    break;
                case "7":
                    str = "七";
                    break;
                case "8":
                    str = "八";
                    break;
                case "9":
                    str = "九";
                    break;
                case "10":
                    str = "十";
                    break;
                case "11":
                    str = "十一";
                    break;
                case "12":
                    str = "十二";
                    break;
                case "一":
                    str = "1";
                    break;
                case "二":
                    str = "2";
                    break;
                case "三":
                    str = "3";
                    break;
                case "四":
                    str = "4";
                    break;
                case "五":
                    str = "5";
                    break;
                case "六":
                    str = "6";
                    break;
                case "七":
                    str = "7";
                    break;
                case "八":
                    str = "8";
                    break;
                case "九":
                    str = "9";
                    break;
                case "十":
                    str = "10";
                    break;
                case "十一":
                    str = "11";
                    break;
                case "十二":
                    str = "12";
                    break;
            }
            return str;
        }

        public GoogleApiFormat GetDirectionByGoogleApiFormat(string Origin, string Destination)
        {
            GoogleApiFormat rt = new GoogleApiFormat();

            string Time = string.Empty;
            string Distance = string.Empty;
            int GetGoogleResult = 0;
            int GetDistResult = 0;
            JObject Result = null;
            try
            {
                Result = GetDirectionByGoogleApi(Origin, Destination);
                Time = Result["routes"][0]["legs"][0]["duration"]["text"].ToString().Replace(" ", "").Trim();
                Distance = Result["routes"][0]["legs"][0]["distance"]["text"].ToString().Replace(" ", "").Trim();

                if (Time.Contains("hours"))
                {
                    Time = Time.Replace("hours", "小时");
                }
                if (Time.Contains("hour"))
                {
                    Time = Time.Replace("hour", "小时");
                }
                Time = Time.Replace("mins", "分钟");

                if (Distance.Contains("mi"))
                {
                    var distSp = Regex.Split(Distance, "mi");
                    if (distSp.Length > 0)
                    {
                        Distance = (decimal.Parse(distSp[0]) * conversion).ToString("#0.00") + " 公里";
                    }
                }
                if (Distance.Contains("km"))
                {
                    Distance = Distance.Replace("km", "公里");
                }

                if (Time.Contains("小时"))
                {
                    var xs = Regex.Split(Time, "小时");
                    var xsValue = int.Parse(xs[0]);
                    var fz = Regex.Split(xs[1], "分钟");
                    var fzValue = int.Parse(fz[0]);
                    GetGoogleResult = xsValue * 60;
                    GetGoogleResult += fzValue;

                }
                else if (Time.Contains("分钟"))
                {
                    GetGoogleResult = int.Parse(Regex.Split(Time, "分钟")[0]);
                }
                if (Distance.Contains("公里"))
                {
                    GetDistResult = (int)decimal.Parse(Regex.Split(Distance, "公里")[0]);
                }

                if (GetGoogleResult > 0 && GetGoogleResult % 5 != 0)
                {
                    while (GetGoogleResult % 5 != 0)
                    {
                        GetGoogleResult++;
                    }
                }
                if (GetDistResult % 5 != 0 && GetDistResult > 0)
                {
                    while (GetDistResult % 5 != 0)
                    {
                        GetDistResult++;
                    }
                }

                int H = (GetGoogleResult / 60);
                int m = (GetGoogleResult % 60);
                string TimeStr = string.Empty;
                if (H != 0)
                {
                    if (H < 10)
                    {
                        TimeStr += "0" + H + "小时";
                    }
                    else
                    {
                        TimeStr += H + "小时";
                    }
                }
                if (m < 10)
                {
                    TimeStr += "0" + m + "分钟";
                }
                else
                {
                    TimeStr += m + "分钟";
                }

                Time = TimeStr;
                Distance = GetDistResult.ToString() + "公里";
                rt.Time = Time;
                rt.Distance = Distance;
                rt.isTrue = true;
                rt.GetGoogleResult = GetDistResult;
                rt.GetDistResult = GetDistResult;

            }
            catch (Exception ex)
            {
                Time = "未知！";
                Distance = "未知！";
                rt.Time = Time;
                rt.Distance = Distance;
                rt.isTrue = false;
                rt.GetGoogleResult = 0;
            }

            return rt;
        }


        string GetTimeGwInfo(ref List<Res_OfficialActivities> listgw, DateTime gwTime, string nowCity)
        {
            List<Res_OfficialActivitiesFormat> formatResGw = listgw.Select(x => new Res_OfficialActivitiesFormat
            {
                Address = x.Address,
                Attendees = x.Attendees,
                Client = x.Client,
                Contact = x.Contact,
                Date = x.Date,
                DiId = x.DiId,
                CreateTime = x.CreateTime,
                CreateUserId = x.CreateUserId,
                DeleteTime = x.DeleteTime,
                DeleteUserId = x.DeleteUserId,
                Dresscode = x.Dresscode,
                Id = x.Id,
                IsDel = x.IsDel,
                IsNeedTrans = x.IsNeedTrans,
                IsPay = x.IsPay,
                IsSubmitApproval = x.IsSubmitApproval,
                Job = x.Job,
                language = x.language,
                OfficialForm = x.OfficialForm,
                Remark = x.Remark,
                Setting = x.Setting,
                Tel = x.Tel,
                Time = x.Time,
                Translators = x.Translators,
                Trip = x.Trip,
                Type = x.Type,
                Area = x.Area,
                Country = x.Country,
                Field = x.Field,
                ReqSample = x.ReqSample,
            }).ToList();

            string info = string.Empty;

            List<Res_OfficialActivitiesFormat> dayArr = new List<Res_OfficialActivitiesFormat>();

            Res_OfficialActivitiesFormat gl = null;

            DateTime newGwTime = DateTime.Now;

            foreach (var g in formatResGw)
            {
                var parseTime = g.GetDateTime;
                if (parseTime.Year == gwTime.Year && parseTime.Month == gwTime.Month && parseTime.Day == gwTime.Day)
                {
                    dayArr.Add(g);
                }
            }

            var isFindTimeGw = false;
            foreach (var g in dayArr)
            {
                var thisData = g.GetDateTime;
                if (thisData.Hour == gwTime.Hour && thisData.Minute == gwTime.Hour)
                {
                    isFindTimeGw = true;
                    newGwTime = thisData;
                    info += $"{thisData.ToString("HH:mm")} 拜访{g.Client},联系人{g.Contact},电话{g.Tel}, 地址{g.Address}" +
                   $"着装要求{g.Dresscode},是否需要翻译{(g.IsNeedTrans == null || g.IsNeedTrans == 0 ? "不需要翻译人员" : "需要翻译人员翻译语种为" + g.language)},议题：- ";
                    listgw.Remove(listgw.Find(x => x.Id == g.Id));
                    gl = g;
                    break;
                }
            }

            //查询时间最接近的
            if (!isFindTimeGw)
            {
                var g = dayArr.OrderBy(x =>
                {
                    return (x.GetDateTime - gwTime).Duration();
                }).FirstOrDefault();
                
                if (g != null)
                {
                    var thisData = g.GetDateTime;
                    if ((thisData - gwTime) <  new TimeSpan(1,30,0))
                    {
                        isFindTimeGw = true;
                        newGwTime = thisData;
                        info += $"{thisData.ToString("HH:mm")} 拜访{g.Client},联系人{g.Contact},电话{g.Tel}, 地址{g.Address}" +
                    $"着装要求{g.Dresscode},是否需要翻译{(g.IsNeedTrans == null || g.IsNeedTrans == 0 ? "不需要翻译人员" : "需要翻译人员翻译语种为" + g.language)},议题：- ";
                        listgw.Remove(listgw.Find(x => x.Id == g.Id));
                        gl = g;
                    }
                }
            }

            //判断是否在一个城市
            if (isFindTimeGw && gl != null && !string.IsNullOrEmpty(gl.Area))
            {
                if (gl.Area.Contains(nowCity))
                {

                }
                else
                {
                    var ApiResult = GetDirectionByGoogleApiFormat(nowCity + "市区", gl.Area);
                    info = info.Insert(0, $"{newGwTime.AddHours(-1).ToString("HH:mm")} 搭乘专车前往{gl.Area}(路程约{ApiResult.Distance}，距离约{ApiResult.Time})\r\n");
                }
            }

            return info;
        }
    }

    public class GoogleApiFormat
    {
        public bool isTrue { get; set; } = false;
        public string Time { get; set; }
        public string Distance { get; set; }
        public int GetDistResult {  get; set; }
        public int GetGoogleResult { get; set; }

    }

    public class Res_OfficialActivitiesFormat : Res_OfficialActivities
    {
        public DateTime GetDateTime
        {
            get
            {
                DateTime.TryParse(this.Date, out DateTime Data);
                //if (this.Date.Contains("T16:"))
                //{
                //    Data =  Data.AddDays(1);
                //}
                Regex rg = new Regex("[0-9]+");
                var gwTimeHHMM = this.Time;
                var gwTimeHHMMMatch = rg.Matches(gwTimeHHMM);
                if (gwTimeHHMMMatch.Count > 1)
                {
                    Data = Data.AddHours(int.Parse(gwTimeHHMMMatch[0].Value));
                    Data = Data.AddMinutes(int.Parse(gwTimeHHMMMatch[1].Value));
                }
                return Data;
            }
        }
    }

}



